#!/usr/bin/env node

// src/index.ts
import { execSync } from "child_process";
import fs from "fs";
import { Stream } from "stream";
import { promisify } from "util";
import chalk from "chalk";
import { cosmiconfig } from "cosmiconfig";
import cpy from "cpy";
import got from "got";
import rimraf from "rimraf";
import * as tar from "tar";
import checkForUpdate from "update-check";

// package.json
var package_default = {
  name: "mui-treasury",
  description: "A CLI to clone Material UI blocks into your project.",
  author: "siriwatknp",
  version: "0.1.6",
  homepage: "https://siriwatknp.github.io/mui-treasury/",
  license: "MIT",
  publishConfig: {
    access: "public"
  },
  repository: {
    type: "git",
    url: "https://github.com/siriwatknp/mui-treasury"
  },
  main: "dist/index.js",
  type: "module",
  exports: "./dist/index.js",
  bin: "./dist/index.js",
  files: [
    "dist"
  ],
  scripts: {
    dev: "tsup --watch",
    build: "tsup",
    test: "jest",
    typecheck: "tsc --noEmit",
    storybook: "storybook dev -p 6006",
    "build-storybook": "storybook build",
    "new-block": "cd .hygen && hygen block new",
    "new-hook": "cd .hygen && hygen block hook"
  },
  resolutions: {
    jackspeak: "2.1.1"
  },
  dependencies: {
    "@types/got": "9.6.11",
    "@types/rimraf": "3.0.0",
    "@types/tar": "4.0.4",
    chalk: "4.1.0",
    cmdk: "0.2.0",
    commander: "7.1.0",
    cosmiconfig: "8.2.0",
    cpy: "8.1.1",
    got: "11.8.2",
    "react-dropzone": "^14.2.3",
    "react-ga4": "2.1.0",
    rimraf: "3.0.2",
    tar: "6.1.0",
    "update-check": "1.5.4"
  },
  devDependencies: {
    "@babel/core": "7.22.9",
    "@babel/preset-env": "7.22.9",
    "@babel/preset-react": "7.22.5",
    "@babel/preset-typescript": "7.22.5",
    "@emotion/react": "11.11.1",
    "@emotion/styled": "11.11.0",
    "@ianvs/prettier-plugin-sort-imports": "4.1.0",
    "@mui/icons-material": "5.15.3",
    "@mui/material": "5.15.1",
    "@storybook/addon-essentials": "7.2.3",
    "@storybook/addon-info": "5.3.21",
    "@storybook/addon-links": "7.2.3",
    "@storybook/addon-mdx-gfm": "7.2.3",
    "@storybook/addon-styling": "1.3.6",
    "@storybook/addons": "7.2.3",
    "@storybook/react": "7.2.3",
    "@storybook/react-webpack5": "7.2.3",
    "@testing-library/jest-dom": "6.1.6",
    "@testing-library/react": "14.1.2",
    "@testing-library/user-event": "14.5.2",
    "@types/color": "3.0.6",
    "@types/jest": "29.5.3",
    "@types/react": "18.2.15",
    "@types/react-dom": "18.2.7",
    "babel-loader": "9.1.3",
    chromatic: "6.24.1",
    color: "4.2.3",
    formik: "2.4.5",
    hygen: "6.2.11",
    jest: "29.6.1",
    "jest-environment-jsdom": "29.7.0",
    react: "18.2.0",
    "react-dom": "18.2.0",
    storybook: "7.2.3",
    "storybook-dark-mode": "3.0.1",
    "ts-jest": "29.1.1",
    "ts-node": "10.9.1",
    tslib: "2.6.0",
    tsup: "7.1.0",
    typescript: "5.1.6",
    "usehooks-ts": "2.9.1",
    yup: "1.3.3"
  }
};

// src/program.ts
import commander from "commander";
function parseTemplate(value) {
  if (value !== "javascript" && value !== "typescript") {
    throw new commander.InvalidOptionArgumentError(
      'only "javascript" or "typescript" is supported via template, pass nothing to use typescript template'
    );
  }
  return value;
}
var createProgram = ({ commands: { clone, init } }) => {
  const program2 = new commander.Command(package_default.name).version(
    package_default.version,
    "-v, --version",
    "output the current version"
  );
  program2.command("init").action(() => {
    init?.();
  });
  program2.command("clone <sources...>").description("clone blocks to your config directory").option("-d, --dir [directory]", "destination directory").option(
    "-t, --template [template]",
    "template of the files, typescript (default) | javascript",
    parseTemplate
  ).option("-b, --branch [branch]", "target branch on github").option("--storybook", "storybook file(s) will be included.").action((sources, options, command) => {
    clone?.(sources, options, command);
  });
  return program2;
};

// src/index.ts
var pipeline = promisify(Stream.pipeline);
var TEMPLATE_FOLDER_MAP = {
  typescript: "",
  javascript: "javascript"
};
var MUI_TREASURY_CONFIG_FILE = "mui-treasury.config.js";
var DEFAULT_CONFIG = {
  dir: "src/mui-treasury",
  template: "typescript",
  storybook: false,
  branch: "main"
};
var CONFIG_FILE_TEMPLATE = `export default {
  dir: "${DEFAULT_CONFIG.dir}",
  template: "${DEFAULT_CONFIG.template}",
  storybook: ${DEFAULT_CONFIG.storybook},
  branch: "${DEFAULT_CONFIG.branch}",
};
`;
var cloneParams = {
  sources: [],
  options: void 0
};
var logger = {
  log: (...text) => {
    console.log(chalk.bgHex("D4D4D8").hex("3F3F46")("mui-treasury"), ...text);
  },
  info: function(text) {
    this.log(
      chalk.bold(chalk.green("info")),
      typeof text === "function" ? text(chalk) : text
    );
  },
  config: function(text) {
    this.log(
      chalk.bold(chalk.hex("0284C7")("config")),
      typeof text === "function" ? text(chalk) : text
    );
  },
  success: function(text) {
    this.log(
      chalk.bold(chalk.green("success")),
      typeof text === "function" ? text(chalk) : text
    );
  },
  version: function() {
    this.log(
      chalk.bold(chalk.hex("F59E0B")("version")),
      chalk.bold(chalk.yellow(`v${package_default.version}`))
    );
  }
};
async function getConfigFile(overrides) {
  let config = DEFAULT_CONFIG;
  try {
    const explorer = cosmiconfig("mui-treasury");
    const result = await explorer.search();
    if (result?.config) {
      config = result.config;
    }
    logger.version();
    logger.info("using config from mui-treasury.config.js");
  } catch (error) {
    logger.info("config file not found, use default config");
    logger.info(
      chalk.blue('{ dir: "src/mui-treasury", storybook: false, test: false }')
    );
  }
  return { ...config, ...overrides };
}
function downloadAndExtractCode(root, sources, branch) {
  return pipeline(
    got.stream(
      `https://codeload.github.com/siriwatknp/mui-treasury/tar.gz/${branch}`
    ),
    tar.extract(
      { cwd: root, strip: 2 },
      sources.map((src) => `mui-treasury-${branch}/blocks/${src}`)
    )
  );
}
function removeDir(dir) {
  return new Promise((resolve, reject) => {
    rimraf(dir, (error) => {
      if (!error) {
        resolve(void 0);
      } else {
        reject(error);
      }
    });
  });
}
var update = checkForUpdate(package_default).catch(() => null);
function shouldUseYarn() {
  try {
    const userAgent = process.env.npm_config_user_agent;
    if (userAgent) {
      return Boolean(userAgent && userAgent.startsWith("yarn"));
    }
    execSync("yarnpkg --version", { stdio: "ignore" });
    return true;
  } catch (e) {
    return false;
  }
}
async function notifyUpdate() {
  try {
    const res = await update;
    if (res?.latest) {
      const isYarn = shouldUseYarn();
      console.log();
      console.log(
        chalk.yellow.bold("A new version of `mui-treasury` is available!")
      );
      console.log(
        "You can update by running: " + chalk.cyan(
          isYarn ? "yarn global add mui-treasury" : "npm i -g mui-treasury"
        )
      );
      console.log();
    }
    process.exit();
  } catch {
  }
}
var program = createProgram({
  commands: {
    clone: (sources, options) => {
      cloneParams.sources = sources;
      cloneParams.options = options;
    },
    init: () => {
      fs.writeFile(
        MUI_TREASURY_CONFIG_FILE,
        CONFIG_FILE_TEMPLATE,
        function(err) {
          if (err)
            throw err;
          logger.success(`${MUI_TREASURY_CONFIG_FILE} is created! \u{1F389}`);
        }
      );
    }
  }
});
program.parse(process.argv);
async function runCloneCommand() {
  const config = await getConfigFile(cloneParams.options);
  if (config.dir && !config.dir.startsWith("/")) {
    config.dir = `/${config.dir}`;
  }
  for (let field of Object.entries(config)) {
    logger.config(`"${field[0]}: ${field[1]}"`);
  }
  const tempRoot = process.cwd() + "/mui-treasury-tmp";
  const tempTemplateRoot = process.cwd() + "/mui-treasury-template-tmp";
  const actualRoot = process.cwd() + config.dir;
  if (!fs.existsSync(tempRoot)) {
    fs.mkdirSync(tempRoot, { recursive: true });
  }
  if (!fs.existsSync(tempTemplateRoot)) {
    fs.mkdirSync(tempTemplateRoot, { recursive: true });
  }
  const sources = cloneParams.sources;
  logger.info(`start cloning ${chalk.bold(sources.length)} packages...`);
  try {
    if (sources.length) {
      await downloadAndExtractCode(tempRoot, sources, config.branch);
    }
    const excludedFiles = [
      ...!config.storybook ? [`!${tempRoot}/**/*.stories.*`] : []
    ];
    logger.info("finishing things up...");
    await Promise.all(
      sources.map(
        (mod) => cpy(
          [
            // default template is typescript (ts codes live in "src" folder)
            `${tempRoot}/${mod}${TEMPLATE_FOLDER_MAP[config.template] ? `/${TEMPLATE_FOLDER_MAP[config.template]}` : ""}/*`,
            ...excludedFiles
          ],
          `${actualRoot}/${mod}`,
          {
            overwrite: true
          }
        )
      )
    );
  } catch (error) {
    logger.log(chalk.bold(chalk.red("\u274C clone failed!")));
    throw error;
  } finally {
    await Promise.all([removeDir(tempRoot), removeDir(tempTemplateRoot)]);
  }
  logger.log(chalk.bold(chalk.green("\u2705 cloned successfully!")));
}
if (cloneParams.sources.length) {
  runCloneCommand().then(notifyUpdate).catch((error) => {
    throw error;
  });
}
//# sourceMappingURL=index.js.map